#cython: language_level=3

from .capi.obidms       cimport OBIDMS_p
from .capi.obidmscolumn cimport OBIDMS_column_p
from .capi.obiview      cimport Obiview_p
from .capi.obitypes     cimport obiversion_t, OBIType_t, index_t
from ._obitaxo          cimport OBI_Taxonomy


cdef class OBIDMS_column:

    cdef  str                _alias               # associated property: alias
    cdef  OBIDMS_column_p*   _pointer
    cdef  OBIView            _view
        
    cpdef close(self)
    
    @staticmethod
    cdef  object  get_subclass_type(OBIDMS_column_p column_p)


cdef class OBIDMS_column_multi_elts(OBIDMS_column):
    
    cpdef set_line(self, index_t line_nb, dict values)


cdef class OBIDMS_column_line:

    cdef OBIDMS_column _column
    cdef index_t       _index


cdef class OBIView:
    
    cdef Obiview_p _pointer
    cdef dict      _columns
    
    cdef Obiview_p _open_or_create_view(self,
                                         OBIDMS dms, 
                                         str view_name, 
                                         bint new=*, 
                                         object view_to_clone=*, 
                                         index_t* line_selection_p=*, 
                                         str comments=*, 
                                         bint quality_column=*, 
                                         str view_type=*)

    cpdef delete_column(self, str column_name)
    cpdef add_column(self, 
                     str column_name, 
                     obiversion_t version_number=*, 
                     str alias=*,
                     str type=*,
                     index_t nb_lines=*, 
                     index_t nb_elements_per_line=*,
                     list elements_names=*,
                     str indexer_name=*,
                     str associated_column_name=*,
                     obiversion_t associated_column_version=*,
                     str comments=*,
                     bint create=*
                    )
    cpdef change_column_alias(self, str current_alias, str new_alias)
    cpdef update_column_pointers(self)
    cpdef close(self)
    

cdef class OBIView_NUC_SEQS(OBIView):
    
    cpdef align(self,
                OBIView oview, 
                OBIView iview2=*, 
                double threshold=*, 
                bint normalize=*, 
                int reference=*, 
                bint similarity_mode=*
               )


cdef class OBIView_line :

    cdef index_t _index
    cdef OBIView _view


cdef class OBIView_line_selection(list):
    
    cdef OBIView _view
    cdef str     _view_name
    
        
cdef class OBIDMS:

    cdef OBIDMS_p _pointer
    
    cpdef close(self)
    cpdef OBI_Taxonomy open_taxonomy(self, str taxo_name)
    cpdef OBIView open_view(self, str view_name)
    cpdef OBIView new_view(self, str view_name, object view_to_clone=*, OBIView_line_selection line_selection=*, str view_type=*, str comments=*, bint quality_column=*)
    cpdef dict read_view_infos(self, str view_name)
#    cpdef dict read_views(self)    TODO

