#cython: language_level=3
 
from .capi.obiview      cimport obi_get_qual_char_with_elt_name_and_col_p_in_view, \
                                obi_get_qual_char_with_elt_idx_and_col_p_in_view, \
                                obi_set_qual_char_with_elt_name_and_col_p_in_view, \
                                obi_set_qual_char_with_elt_idx_and_col_p_in_view, \
                                obi_get_qual_int_with_elt_name_and_col_p_in_view, \
                                obi_get_qual_int_with_elt_idx_and_col_p_in_view, \
                                obi_set_qual_int_with_elt_name_and_col_p_in_view, \
                                obi_set_qual_int_with_elt_idx_and_col_p_in_view
                                
from .capi.obierrno     cimport obi_errno
from .capi.obitypes     cimport OBIQual_char_NA, OBIQual_int_NA, const_char_p

from ._obidms           cimport OBIView

from obitools3.utils cimport str2bytes, bytes2str

from libc.stdlib cimport free
from libc.string cimport strcmp
from libc.stdint cimport uint8_t
from libc.stdlib cimport malloc


cdef class OBIDMS_column_qual(OBIDMS_column):

    cpdef object get_line(self, index_t line_nb):
        cdef const uint8_t*  value
        cdef int             value_length
        cdef object          result
        cdef int             i
        value = obi_get_qual_int_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, &value_length)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBIQual_int_NA :
            result = None
        else :
            result = []
            for i in range(value_length) :
                result.append(<int>value[i])            
        return result
    
    cpdef object get_str_line(self, index_t line_nb):
        cdef char*  value
        cdef object result
        cdef int    i
        value = obi_get_qual_char_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBIQual_char_NA :
            result = None
        else :
            result = bytes2str(value)
            free(value)
        return result

    cpdef set_line(self, index_t line_nb, object value):
        cdef uint8_t* value_b
        cdef int      value_length
        if value is None :
            if obi_set_qual_int_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, OBIQual_int_NA, 0) < 0:
                raise Exception("Problem setting a value in a column")
        else :
            value_length = len(value)
            value_b = <uint8_t*> malloc(value_length * sizeof(uint8_t))
            for i in range(value_length) :
                value_b[i] = <uint8_t>value[i]
            if obi_set_qual_int_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, value_b, value_length) < 0:
                raise Exception("Problem setting a value in a column")
            free(value_b)

    cpdef set_str_line(self, index_t line_nb, object value):
        cdef bytes value_b        
        if value is None :
            if obi_set_qual_char_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, OBIQual_char_NA) < 0:
                raise Exception("Problem setting a value in a column")
        else :
            if type(value) == str:
                value_b = str2bytes(value)
            else :
                value_b = value
            if obi_set_qual_char_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, value_b) < 0:
                raise Exception("Problem setting a value in a column")


cdef class OBIDMS_column_multi_elts_qual(OBIDMS_column_multi_elts):
 
    cpdef object get_item(self, index_t line_nb, str element_name):
        cdef const uint8_t*  value
        cdef int             value_length
        cdef object          result
        cdef int             i
        value = obi_get_qual_int_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), &value_length)
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBIQual_int_NA :
            result = None
        else :
            result = []
            for i in range(value_length) :
                result.append(<int>value[i])            
        return result
 
    cpdef object get_str_item(self, index_t line_nb, str element_name):
        cdef char*  value
        cdef object result
        value = obi_get_qual_char_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name))
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBIQual_char_NA :
            result = None
        else :
            result = bytes2str(value)
            free(value)
        return result

    cpdef object get_line(self, index_t line_nb) :
        cdef const uint8_t*  value
        cdef int             value_length
        cdef object          value_in_result
        cdef dict            result
        cdef index_t         i
        cdef int             j
        cdef bint            all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_qual_int_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, i, &value_length)
            if obi_errno > 0 :
                raise IndexError(line_nb)
            if value == OBIQual_int_NA :
                value_in_result = None
            else :
                value_in_result = []
                for j in range(value_length) :
                    value_in_result.append(<int>value[j])            
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result
 
    cpdef object get_str_line(self, index_t line_nb) :
        cdef char*   value
        cdef object  value_in_result
        cdef dict    result
        cdef index_t i
        cdef bint    all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_qual_char_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, i)
            if obi_errno > 0 :
                raise IndexError(line_nb)
            if value == OBIQual_char_NA :
                value_in_result = None
            else :
                value_in_result = bytes2str(value)
                free(value)
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result

    cpdef set_item(self, index_t line_nb, str element_name, object value):
        cdef uint8_t* value_b
        cdef int      value_length
        if value is None :
            if obi_set_qual_int_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), OBIQual_int_NA, 0) < 0:
                raise Exception("Problem setting a value in a column")
        else :
            value_length = len(value)
            value_b = <uint8_t*> malloc(value_length * sizeof(uint8_t))
            for i in range(value_length) :
                value_b[i] = <uint8_t>value[i]
            if obi_set_qual_int_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), value_b, value_length) < 0:
                raise Exception("Problem setting a value in a column")
            free(value_b)

    cpdef set_str_item(self, index_t line_nb, str element_name, object value):
        if value is None :
            if obi_set_qual_char_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), OBIQual_char_NA) < 0:
                raise Exception("Problem setting a value in a column")
        else :
            if obi_set_qual_char_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), str2bytes(value)) < 0:
                raise Exception("Problem setting a value in a column")
