#cython: language_level=3
 
from .capi.obiview      cimport obi_get_seq_with_elt_name_and_col_p_in_view, \
                                obi_get_seq_with_elt_idx_and_col_p_in_view, \
                                obi_set_seq_with_elt_name_and_col_p_in_view, \
                                obi_set_seq_with_elt_idx_and_col_p_in_view
from .capi.obialign     cimport obi_align_one_column
from .capi.obierrno     cimport obi_errno
from .capi.obitypes     cimport OBISeq_NA, const_char_p

from ._obidms           cimport OBIView

from obitools3.utils cimport str2bytes, bytes2str

from libc.stdlib cimport free


cdef class OBIDMS_column_seq(OBIDMS_column):

    cpdef object get_line(self, index_t line_nb):
        cdef char*  value
        cdef object result
        value = obi_get_seq_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBISeq_NA :
            result = None
        else :
            try:
                result = <bytes> value
            finally:
                free(value)
        return result

    cpdef set_line(self, index_t line_nb, object value):
        cdef bytes value_b

        if value is None :
            value_b = OBISeq_NA
        elif isinstance(value, bytes) :
            value_b = value
        elif isinstance(value, str) :
            value_b = str2bytes(value)
        else:
            raise TypeError('Sequence value must be of type Bytes, Str or None')

        if obi_set_seq_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, 0, value_b) < 0:
            raise Exception("Problem setting a value in a column")
        

cdef class OBIDMS_column_multi_elts_seq(OBIDMS_column_multi_elts):

    cpdef object get_item(self, index_t line_nb, str element_name):
        cdef char*  value
        cdef object result
        value = obi_get_seq_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name))
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBISeq_NA :
            result = None
        else :
            try:
                result = <bytes> value
            finally:
                free(value)
        return result

    
    cpdef object get_line(self, index_t line_nb) :
        cdef char*   value
        cdef object  value_in_result
        cdef dict    result
        cdef index_t i
        cdef bint    all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_get_seq_with_elt_idx_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, i)
            if obi_errno > 0 :
                raise IndexError(line_nb)
            if value == OBISeq_NA :
                value_in_result = None
            else :
                try:
                    value_in_result = <bytes> value
                finally:
                    free(value)    
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result

    cpdef set_item(self, index_t line_nb, str element_name, object value):
        cdef bytes value_b
        
        if value is None :
            value_b = OBISeq_NA
        elif isinstance(value, bytes) :
            value_b = value
        elif isinstance(value, str) :
            value_b = str2bytes(value)
        else:
            raise TypeError('Sequence value must be of type Bytes, Str or None')
        
        if obi_set_seq_with_elt_name_and_col_p_in_view(self._view._pointer, (self._pointer)[0], line_nb, str2bytes(element_name), value_b) < 0:
            raise Exception("Problem setting a value in a column")

